--[[
##############################################
##-
##-  WakeSpams
##-  By Cyrila <Aspire> @ Bloodfeather-EU
##-
##-  Cooldowns For Your Raid
##-
##-
##-   * Copyright (C) 2009  Dan Jacobsen
##-   *
##-   * This program is free software: you can redistribute it and/or modify
##-   * it under the terms of the GNU General Public License as published by
##-   * the Free Software Foundation, either version 3 of the License, or
##-   * (at your option) any later version.
##-   *
##-   * This program is distributed in the hope that it will be useful,
##-   * but WITHOUT ANY WARRANTY; without even the implied warranty of
##-   * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##-   * GNU General Public License for more details.
##-   *
##-   * You should have received a copy of the GNU General Public License
##-   * along with this program.  If not, see <http://www.gnu.org/licenses/>.
##-
##############################################
]]

local WakeSpams = LibStub("AceAddon-3.0"):NewAddon("WakeSpams", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
_G["WakeSpams"] = WakeSpams

local pairs = pairs;
local ipairs = ipairs;
local sin = _G.math.sin;
local floor = _G.math.floor;
local ceil = _G.math.ceil;
local tinsert = tinsert;
local tremove = tremove;
local tsort = _G.table.sort;
local strlen = _G.string.len;
local strsub = _G.string.sub;
local strfind = _G.string.find;
local strformat = _G.string.format;
local wipe = wipe;

local GetSpellInfo = _G["GetSpellInfo"];
local GetTime = _G["GetTime"];

WakeSpams.Msg = nil
WakeSpams.MsgFrame = "SELF"
WakeSpams.Timer = {}
WakeSpams.LastSent = {0, "", ""}
WakeSpams.RaidIcons = {
	[1] = "{star} ", -- Yellow 4-point Star
	[2] = "{circle} ", -- Orange Circle
	[3] = "{diamond} ", -- Purple Diamond
	[4] = "{triangle} ", -- Green Triangle
	[5] = "{moon} ", -- White Crescent Moon
	[6] = "{square} ", -- Blue Square
	[7] = "{cross} ", -- Red 'X' Cross
	[8] = "{skull} "  -- White Skull
}
WakeSpams.Queue = {}

-- --------------------------------
--  DEFAULTS
-- --------------------------------

local db
local defaults = {
	profile = {
		enabled = true,
		Interval = 10000,
		aoe = true,
		AbilityStart = "Activated: #s# #d#s!",
		AbilityEnd = "Faded: #s#",
		SpellList = {
		},
		EventList = {
		},
	},
}

function WakeSpams:OnInitialize()
	-- Init
	self.db = LibStub("AceDB-3.0"):New("WakeSpamsDB", defaults, "Default")
	db = self.db.profile
	
	-- Register spells
	
	self:RegisterChatCommand("wakespams", "SlashHandler");
end

function WakeSpams:OnEnable()
	-- ENABLED
	self:Print(" -- WakeSpams Loaded! you must look, you must tab, you must see... future. -- ");
	
	-- Register events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "COMBAT_LOG_EVENT_UNFILTERED");
	self:RegisterEvent("RAID_ROSTER_UPDATE", "RAID_ROSTER_UPDATE");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "RAID_ROSTER_UPDATE");
	--self:RegisterEvent("PLAYER_ENTERING_WORLD", "RAID_ROSTER_UPDATE");
end

function WakeSpams:OnDisable()
	-- Do something...
end


-- ----------------------------
--  Config
-- ----------------------------


function WakeSpams:SlashHandler(input)
	self.Options:ShowConfig()
	--[[
	local newsetting
	input = self:Explode(" ", string.lower(input))
	if(input[1] == "enable") then
		if(db.STANDBY == 0) then
			self:Print("Combat log monitoring is already enabled.")
		else
			db.STANDBY = 0
			self:Print("Combat log monitoring enabled.")
		end
	elseif(input[1] == "disable") then
		if(db.STANDBY == 1) then
			self:Print("Combat log monitoring is already disabled.")
		else
			db.STANDBY = 1
			self:Print("Combat log monitoring disabled.")
		end
	elseif(input[1] == "aoe") then
		db.aoe = (db.aoe == 0) and 1 or 0
		newsetting = (db.aoe == 0) and "Disabled" or "Enabled"
		self:Print(("AoE spell announcing has been >%s<"):format(newsetting))
	elseif(input[1] == "settings") then
		self.Options:ShowConfig()
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFCC00WakeSpams help:")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC9900   Enable|r - Starts combat log monitoring")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC9900   Disable|r - Stops combat log monitoring")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC9900   Aoe|r - Toggles announcing multitarget spells (such as Shockwave and Howl of Terror)")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC9900   Help|r - Shows help")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFCC1100   This addon has no config. You can add and edit spell events in the Lua file.|r")
	end
	]]
end


-- ------------------------------
--  Stuff
-- ------------------------------

function WakeSpams:RaidIconText(t)
	local ri = ""
	if UnitExists(t) then
		ri = self.RaidIcons[GetRaidTargetIndex(t)] or ri
	end
	return ri
end

function WakeSpams:GetUnitID(GUID, name)
	-- Translates GUID to target (target, focus, mouseover, playername)
	if UnitGUID("focus") == GUID then
		return "focus"
	end
	if UnitGUID("mouseover") == GUID then
		return "mouseover"
	end
	if UnitGUID("target") == GUID then
		return "target"
	end
	return name
end

-- ------------------------------------------------------------
--  HANDLE COMBAT LOG EVENTS AND PASS TO REGISTER SPELL OBJECT
-- ------------------------------------------------------------
function WakeSpams:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, sourceGUID, sourceName, _, destGUID, destName, _, spellid, spellname, _, arg1, arg2)
	-- Nil
	if not db.enabled then return nil end
	if(sourceGUID ~= UnitGUID("player")) then return nil end
	
	local spell
	
	-- Prioritize catchall events
	if(self.SpellEvent[event]) then
		if(db.EventList[event]) then
			if(db.EventList[event].disabled) then
				return nil
			end
		end
		self.Msg = self.SpellEvent[event].txt or db.AbilityStart
	elseif(spellid) then
		if not self.SpellList then return nil end
		if(self.SpellList[spellid]) then
			spell = self.SpellList[spellid]
			if(db.SpellList[spellid]) then
				profile = db.SpellList[spellid]
				if(profile.disabled) then return nil end
				spell.txt = profile.txt or spell.txt
				spell.whisper = profile.whisper or spell.whisper
			end
			if not db.aoe and (spell.aoe) then return nil end
			if not spell.event and (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REMOVED") then
				-- SPELL_AURA
				if(event == "SPELL_AURA_REMOVED") then
					if not spell.countdown or (spell.countdown < 0) then
						return nil
					end
					if(spell.countdown > 0) then
						-- Stop timers
						self:CancelTimer(self.Timer[spellid].timer, true)
						self.Timer[spellid] = nil
					end
					self.Msg = spell.txtend or db.AbilityEnd
				else
					self.Msg = spell.txt or db.AbilityStart
					
					if(self:isNumeric(spell.countdown) and spell.countdown > 0) then
						self.Timer[spellid] = {}
						self.Timer[spellid].elapsed = 0
						self.Timer[spellid].timer = self:ScheduleRepeatingTimer("TimerHandle", 1, {spellid, spell.dur, spell.dur-spell.countdown}) -- duration, duration-time to know when to start announcing
					end
				end
			end
			if(spell.event == event) then
				self.Msg = spell.txt or db.AbilityStart
			end
		end
	end
	
	-- Return nil if no message was formatted
	if not self.Msg then return nil end
	
	if(spell) then
		if(spell.aoe) then
			self:Enqueue(spellid, destName)
			self.Msg = nil
			return true
		end
	end
	spellname = spell.name or GetSpellLink(spellid)
	msgframe = spell.output or self.MsgFrame
	
	-- Format string
	self.Msg = string.gsub(self.Msg, "#(.-)#", function(a)
		if(a == "s") then
			a = spellname
		elseif(a == "maxhp") then
			a = floor(UnitHealthMax("player"))
		elseif(a == "halfhp") then
			a = floor(UnitHealthMax("player")*0.5)
		elseif(a == "d") then
			a = spell.dur
		elseif(a == "amount") then
			a = arg1
		elseif(a == "extra") then
			if(event == "SPELL_MISSED") then
				a = tostring(arg1)
			else
				a = arg2
			end
		elseif(a == "extra:SpellLink" and event ~= "SPELL_MISSED") then
			a = GetSpellLink(arg1)
		elseif(a == "t") then
			if(destName == UnitName("player")) then
				a = "me"
			else
				a = GetUnitName(destName, nil) or destName
				a = self:RaidIconText(self:GetUnitID(destGUID, destName))..a
			end
		end
		return a
	end)
	
	if(spell.whisper) then
		if(destName == UnitName("player")) then
			self:ChatMSG(self.Msg, "SELF")
		else
			self:ChatMSG(self.Msg, "WHISPER", nil, destName);
		end
	else
		self:ChatMSG(self.Msg, msgframe);
	end
	-- Set msg to nil, otherwise we might have "some" spam issues.
	self.Msg = nil
end

function WakeSpams:Enqueue(spellID, destName)
	if not self.Queue[spellID] then
		-- Create new timer
		self:ScheduleTimer("SendQueue", 0.1, spellID)
		self.Queue[spellID] = {}
	end
	tinsert(self.Queue[spellID], {destName});
end

function WakeSpams:SendQueue(spellID)
	if self.Queue[spellID] then
		local total
		local targetN
		for i,v in ipairs(self.Queue[spellID]) do
			total = i
			targetN = (targetN) and (targetN..", "..v[1]) or v[1]
		end
		local msg
		if(total == 1) then
			msg = (("%s: Hit %d target! (%s)"):format(GetSpellLink(spellID),total,targetN))
		else
			msg = (("%s: Hit %d targets!"):format(GetSpellLink(spellID),total))
		end
		--local msg = "SpellLink: Hit 3 targets (X, Y, Z)"
		self:ChatMSG(msg, self.MsgFrame)
		self.Queue[spellID] = nil
	end
end

function WakeSpams:RAID_ROSTER_UPDATE()
	if(GetNumRaidMembers() > 1) then
		self.MsgFrame = "RAID"
	elseif(GetNumRaidMembers() <= 0 and GetNumPartyMembers() >= 1) then
		self.MsgFrame = "PARTY"
	else
		self.MsgFrame = "SELF"
	end
	--self:ChatMSG("Raid Roster Update ("..GetNumRaidMembers().." raid members and "..GetNumPartyMembers().." party members). Now sending messages to: "..self.MsgFrame, self.MsgFrame)
end

function WakeSpams:TimerHandle(args)
	-- args: 1 = spellid, 2 = d, 3 = ts
	self.Timer[args[1]].elapsed = self.Timer[args[1]].elapsed + 1
	
	if(self.Timer[args[1]].elapsed >= args[3] and self.Timer[args[1]].elapsed < args[2]) then
		self:ChatMSG(""..args[2] - self.Timer[args[1]].elapsed.."", self.MsgFrame)
	end
end

function WakeSpams:ChatMSG(msg, mtype, mlang, marg)
	if(GetTime() < self.LastSent[1]+db.Interval and self.LastSent[2] == msg and self.LastSent[3] == mtype) then -- Temporary chat throttle
		return nil
	end
	if(mtype=="SELF") then
		DEFAULT_CHAT_FRAME:AddMessage(msg)
	else
		SendChatMessage(msg, mtype, mlang, marg)
	end
	self.LastSent = {GetTime(), msg, mtype}
end

function WakeSpams:isNumeric(a)
    return type(tonumber(a)) == "number"
end

function WakeSpams:ApplyConstant(array)
	return setmetatable ({}, 
	{
		__index = array,
		__newindex = function (t, n, v)
			self:Print("Error: attempting to change constant "..tostring(n).." to "..tostring(v))
		end
    })
end

function WakeSpams:Explode(d, str)
	local t, ll
	t={}
	ll=0
	if(#str == 1) then return str end
		while true do
			l=strfind(str, d, ll+1, true)
			if l~=nil then
				tinsert(t, strsub(str, ll, l-1))
				ll=l+1
			else
				tinsert(t, strsub(str,ll))
				break
			end
		end
	return t
end