local addon = WakeSpams
local module = addon:NewModule("Options")
WakeSpams.Options = module

local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
local db

local options

function module:RegisterOptions()
	-- xD
	options = {
		name = 'WakeSpams',
		type = "group",
		args = {
			general = {
				name = 'WakeSpams',
				type = "group",
				args = {
					enabled = {
						order = 1,
						type = "toggle",
						name = "Enabled",
						desc = "Enable or disable addon.",
						descStyle = "inline",
						set = function(v) db.profile.enabled = not db.profile.enabled end,
						get = function() return db.profile.enabled end,
					},
					spellheader = {
						name = "Spell Announcements",
						type = "header",
						order = 10,
					},
					aoe = {
						order = 11,
						type = "toggle",
						name = "AoE",
						desc = "Enable or disable announcements for multitarget spells.",
						descStyle = "inline",
						set = function(v) db.profile.aoe = not db.profile.aoe end,
						get = function() return db.profile.aoe end,
					},
					interrupts = {
						order = 12,
						type = "toggle",
						name = "Interrupts",
						desc = "Enable or disable announcements for interrupts.",
						descStyle = "inline",
						set = function(i,v)
							if not db.profile.EventList["SPELL_INTERRUPT"] then
								db.profile.EventList["SPELL_INTERRUPT"] = {}
							end
							db.profile.EventList["SPELL_INTERRUPT"].disabled = not v
						end,
						get = function()
							if not db.profile.EventList["SPELL_INTERRUPT"] then
								return true
							else
								return not db.profile.EventList["SPELL_INTERRUPT"].disabled
							end
						end,
					},
					dispelresists = {
						order = 13,
						type = "toggle",
						name = "Dispel Resists",
						desc = "Enable or disable announcements for dispel resists.",
						descStyle = "inline",
						set = function(i,v)
							if not db.profile.EventList["SPELL_DISPEL_FAILED"] then
								db.profile.EventList["SPELL_DISPEL_FAILED"] = {}
							end
							db.profile.EventList["SPELL_DISPEL_FAILED"].disabled = not v
						end,
						get = function()
							if not db.profile.EventList["SPELL_DISPEL_FAILED"] then
								return true
							else
								return not db.profile.EventList["SPELL_DISPEL_FAILED"].disabled
							end
						end,
					},
					defaults = {
						name = "Edit Defaults",
						type = "header",
						order = 20,
					},
					AbilityStart = {
						name = "",
						type = "group",
						inline = true,
						order = 21,
						args = {
							input = {
								order = 3,
								type = "input",
								name = "Default ability start text",
								desc = "#s# #d# etc xDD",
								width = "full",
								set = function(i,v) db.profile.AbilityStart = v end,
								get = function() return db.profile.AbilityStart end,
							},
						},
					},
					AbilityEnd = {
						name = "",
						type = "group",
						inline = true,
						order = 22,
						args = {
							input = {
								order = 3,
								type = "input",
								name = "Default ability end text",
								desc = "#s# #d# etc xDD",
								width = "full",
								set = function(i,v) db.profile.AbilityEnd = v end,
								get = function() return db.profile.AbilityEnd end,
							},
						},
					},
					strreplace = {
						name = "String replacements",
						type = "header",
						order = 30,
					},
					strreplacedesc = {
						name = "#s# = spellname,\
						#d# = duration,\
						#t# = targetname,\
						#amount# = amount (heal, damage, etc),\
						#maxhp# = your maximum HP,\
						#halfhp# = half of your maximum HP,\
						#extra# = resist type for resists, spell names for interrupted spells, etc.\
						#extra:Spellname# = makes spell link of extra arg if available.",
						type = "description",
						fontSize = "medium",
						order = 31,
					},
				},
			},
		},
	}
	
	local function EditSpell(spellid)
		local sname, _, sicon, _, _, _, _, _, _ = GetSpellInfo(spellid)
		local spell = addon.SpellList[spellid]
		if not db.profile.SpellList[spellid] then
			db.profile.SpellList[spellid] = {}
		end
		local tbl = {
			name = "Edit Spell",
			type = "group",
			order = 3,
			args = {
				description = {
					name = sname,
					fontSize = "large",
					type = "description",
					order = 1,
					width = "full",
					imageCoords = {0.1,0.9,0.1,0.9},
					imageWidth = 20,
					imageHeight = 20,
				},
				txt = {
					name = "Edit start message of spell.",
					type = "input",
					order = 2,
					width = "full",
					set = function(info,val) db.profile.SpellList[spellid].txt = val end,
					get = function() return db.profile.SpellList[spellid].txt or spell.txt end,
				},
				buttonOkay = {
					name = "Okay",
					type = "execute",
					order = 3,
					width = "half",
					func = function() options.args.spells.args.edit = nil end,
				},
				buttonDefaults = {
					name = "Defaults",
					type = "execute",
					order = 4,
					width = "half",
					func = function() db.profile.SpellList[spellid] = {} end,
				},
			},
		}
		return tbl
	end
	
	local function PopulateClassOptions()
		local tbl
		tbl = {
			name = 'spells',
			type = "group",
			desc = "Spells",
			childGroups = "tab",
			args = {
				add = {
					name = "Add Spell",
					type = "group",
					desc = "Add spell",
					order = 2,
					args = {
						description = {
							name = "Add Spell",
							type = "description",
							order = 1,
						},
					},
				},
				list = {
					name = "Spell List",
					type = "group",
					desc = "Spell list",
					order = 1,
					args = {
					},
				},
			},
		}
		if not addon.SpellList then
			local class, _ = UnitClass("player");
			tbl.args.list.args.info = {
				type = "description",
				name = ("There are no preconfigured %s spells. To add a spell, click the  \"New Spell\" button."):format(class),
				order = 100,
			}
			return tbl
		end
		for i,v in pairs(addon.SpellList) do
			local sname, _, sicon, _, _, _, _, _, _ = GetSpellInfo(i)
			tbl.args.list.args[""..i..""] = {
				name = sname,
				type = "group",
				inline = true,
				order = i,
				args = {
					description = {
						name = sname,
						type = "description",
						order = 1,
						image = sicon,
						width = "half",
						imageCoords = {0.1,0.9,0.1,0.9},
						imageWidth = 20,
						imageHeight = 20,
					},
					toggleSpell = {
						name = "Enable",
						type = "toggle",
						order = 2,
						width = "half",
						desc = GetSpellLink(i)..' Enable/disable announcing for this spell.',
						--descStyle = "inline",
						set = function(info,val)
							if not db.profile.SpellList[i] then
								db.profile.SpellList[i] = {}
							end
							db.profile.SpellList[i].disabled = not val
						end,
						get = function()
							if not db.profile.SpellList[i] then
								return true
							else
								return not db.profile.SpellList[i].disabled
							end
						end,
					},
					editSpell = {
						name = "Edit",
						type = "execute",
						order = 3,
						width = "half",
						desc = GetSpellLink(i)..' Edit this spell.',
						func = function()
							options.args.spells.args.edit = EditSpell(i)
						end,
					}
				}
			}
		end
		return tbl
	end
	options.args.spells = PopulateClassOptions()
	
	options.args.help = {
		type = "group",
		name = "Help",
		args = {
			strreplace = {
				name = "String replacements",
				type = "header",
				order = 1,
			},
			strreplacedesc = {
				name = "#s# = spellname,\
				#d# = duration,\
				#t# = targetname,\
				#amount# = amount (heal, damage, etc),\
				#maxhp# = your maximum HP,\
				#halfhp# = half of your maximum HP,\
				#extra# = resist type for resists, spell names for interrupted spells, etc.\
				#extra:Spellname# = makes spell link of extra arg if available.",
				type = "description",
				fontSize = "medium",
				order = 2,
			},
		},
	}

	options.args.about = {
		type = "group",
		name = "About",
		order = -1,
		args = {
			author = {
				type = "description",
				name = "Author: |cFFCC9900Cyrila [Bloodfeather-EU]|r\
\
WakeSpams is a spell announcement addon.|r\
\
\
Get the latest WakeSpams update from the project page on WoWAce.com",
				order = 100,
			},
		},
	}
	
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)
end

function module:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.InterfaceOptions)
end

function module:OnInitialize()
	db = addon.db
	self:RegisterOptions()
	ACR:RegisterOptionsTable("WakeSpams", options)
	
	self.InterfaceOptions = ACD:AddToBlizOptions("WakeSpams", "WakeSpams", nil, "general")
	local class, _ = UnitClass("player");
	ACD:AddToBlizOptions("WakeSpams", class, "WakeSpams", "spells")
	ACD:AddToBlizOptions("WakeSpams", "Help", "WakeSpams", "help")
	ACD:AddToBlizOptions("WakeSpams", "Profiles", "WakeSpams", "profiles")
	ACD:AddToBlizOptions("WakeSpams", "About", "WakeSpams", "about")
end